@@ -4,23 +4,19 @@ from __future__ import absolute_import
 import os
 import sys
 
-import crayons
-import delegator
-
 from click import (
-    argument, echo, edit, group, option, pass_context, secho, version_option
+    argument, echo, edit, group, option, pass_context, secho, version_option, Choice
 )
 
-import click_completion
-
-from click_didyoumean import DYMCommandCollection
-
 from ..__version__ import __version__
+from ..patched import crayons
+from ..vendor import click_completion, delegator
 from .options import (
     CONTEXT_SETTINGS, PipenvGroup, code_option, common_options, deploy_option,
-    general_options, install_options, lock_options, pass_state, skip_lock_option,
-    pypi_mirror_option, python_option, requirementstxt_option, sync_options,
-    system_option, three_option, verbose_option, uninstall_options
+    general_options, install_options, lock_options, pass_state,
+    pypi_mirror_option, python_option, site_packages_option, skip_lock_option,
+    sync_options, system_option, three_option, uninstall_options,
+    verbose_option
 )
 
 
@@ -47,7 +43,7 @@ subcommand_context_no_interspersion["allow_interspersed_args"] = False
     "--completion",
     is_flag=True,
     default=False,
-    help="Output completion (to be eval'd).",
+    help="Output completion (to be executed by the shell).",
 )
 @option("--man", is_flag=True, default=False, help="Display manpage.")
 @option(
@@ -64,19 +60,15 @@ def cli(
     state,
     where=False,
     venv=False,
+    py=False,
+    envs=False,
     rm=False,
     bare=False,
-    three=False,
-    python=False,
-    help=False,
-    py=False,
-    site_packages=False,
-    envs=False,
-    man=False,
     completion=False,
-    pypi_mirror=None,
+    man=False,
     support=None,
-    clear=False,
+    help=False,
+    site_packages=None,
     **kwargs
 ):
     # Handle this ASAP to make shell startup fast.
@@ -110,7 +102,7 @@ def cli(
 
     if man:
         if system_which("man"):
-            path = os.sep.join([os.path.dirname(__file__), "pipenv.1"])
+            path = os.path.join(os.path.dirname(os.path.dirname(__file__)), "pipenv.1")
             os.execle(system_which("man"), "man", path, os.environ)
             return 0
         else:
@@ -125,7 +117,7 @@ def cli(
         echo(
             "\nYou can learn more at:\n   {0}".format(
                 crayons.green(
-                    "http://docs.pipenv.org/advanced/#configuration-with-environment-variables"
+                    "https://pipenv.pypa.io/en/latest/advanced/#configuration-with-environment-variables"
                 )
             )
         )
@@ -146,16 +138,19 @@ def cli(
             get_pipenv_diagnostics()
             return 0
         # --clear was passed…
-        elif clear:
+        elif state.clear:
             do_clear()
             return 0
-
         # --venv was passed…
         elif venv:
             # There is no virtualenv yet.
             if not project.virtualenv_exists:
                 echo(
-                    crayons.red("No virtualenv has been created for this project yet!"),
+                    "{}({}){}".format(
+                        crayons.red("No virtualenv has been created for this project"),
+                        crayons.white(project.project_directory, bold=True),
+                        crayons.red(" yet!")
+                    ),
                     err=True,
                 )
                 ctx.abort()
@@ -198,7 +193,7 @@ def cli(
                 )
                 ctx.abort()
     # --two / --three was passed…
-    if (state.python or state.three is not None) or site_packages:
+    if (state.python or state.three is not None) or state.site_packages:
         ensure_project(
             three=state.three,
             python=state.python,
@@ -220,6 +215,7 @@ def cli(
 @system_option
 @code_option
 @deploy_option
+@site_packages_option
 @skip_lock_option
 @install_options
 @pass_state
@@ -241,7 +237,7 @@ def install(
         lock=not state.installstate.skip_lock,
         ignore_pipfile=state.installstate.ignore_pipfile,
         skip_lock=state.installstate.skip_lock,
-        requirements=state.installstate.requirementstxt,
+        requirementstxt=state.installstate.requirementstxt,
         sequential=state.installstate.sequential,
         pre=state.installstate.pre,
         code=state.installstate.code,
@@ -252,21 +248,21 @@ def install(
         extra_index_url=state.extra_index_urls,
         packages=state.installstate.packages,
         editable_packages=state.installstate.editables,
+        site_packages=state.site_packages
     )
     if retcode:
         ctx.abort()
 
 
 @cli.command(
-    short_help="Un-installs a provided package and removes it from Pipfile.",
+    short_help="Uninstalls a provided package and removes it from Pipfile.",
     context_settings=subcommand_context
 )
-@option("--skip-lock/--lock", is_flag=True, default=False, help="Lock afterwards.")
 @option(
     "--all-dev",
     is_flag=True,
     default=False,
-    help="Un-install all package from [dev-packages].",
+    help="Uninstall all package from [dev-packages].",
 )
 @option(
     "--all",
@@ -280,12 +276,11 @@ def install(
 def uninstall(
     ctx,
     state,
-    skip_lock=False,
     all_dev=False,
     all=False,
     **kwargs
 ):
-    """Un-installs a provided package and removes it from Pipfile."""
+    """Uninstalls a provided package and removes it from Pipfile."""
     from ..core import do_uninstall
     retcode = do_uninstall(
         packages=state.installstate.packages,
@@ -303,6 +298,20 @@ def uninstall(
     if retcode:
         sys.exit(retcode)
 
+LOCK_HEADER = """\
+#
+# These requirements were autogenerated by pipenv
+# To regenerate from the project's Pipfile, run:
+#
+#    pipenv lock {options}
+#
+"""
+
+LOCK_DEV_NOTE="""\
+# Note: in pipenv 2020.x, "--dev" changed to emit both default and development
+# requirements. To emit only development requirements, pass "--dev-only".
+"""
+
 @cli.command(short_help="Generates Pipfile.lock.", context_settings=CONTEXT_SETTINGS)
 @lock_options
 @pass_state
@@ -314,22 +323,51 @@ def lock(
 ):
     """Generates Pipfile.lock."""
     from ..core import ensure_project, do_init, do_lock
-
     # Ensure that virtualenv is available.
-    ensure_project(three=state.three, python=state.python, pypi_mirror=state.pypi_mirror)
-    if state.installstate.requirementstxt:
+    # Note that we don't pass clear on to ensure_project as it is also
+    # handled in do_lock
+    ensure_project(
+        three=state.three, python=state.python, pypi_mirror=state.pypi_mirror,
+        warn=(not state.quiet), site_packages=state.site_packages,
+    )
+    emit_requirements = state.lockoptions.emit_requirements
+    dev = state.installstate.dev
+    dev_only = state.lockoptions.dev_only
+    pre = state.installstate.pre
+    if emit_requirements:
+        # Emit requirements file header (unless turned off with --no-header)
+        if state.lockoptions.emit_requirements_header:
+            header_options = ["--requirements"]
+            if dev_only:
+                header_options.append("--dev-only")
+            elif dev:
+                header_options.append("--dev")
+            click.echo(LOCK_HEADER.format(options=" ".join(header_options)))
+            # TODO: Emit pip-compile style header
+            if dev and not dev_only:
+                click.echo(LOCK_DEV_NOTE)
+        # Setting "emit_requirements=True" means do_init() just emits the
+        # install requirements file to stdout, it doesn't install anything
         do_init(
-            dev=state.installstate.dev,
-            requirements=state.installstate.requirementstxt,
+            dev=dev,
+            dev_only=dev_only,
+            emit_requirements=emit_requirements,
             pypi_mirror=state.pypi_mirror,
             pre=state.installstate.pre,
         )
+    elif state.lockoptions.dev_only:
+        raise exceptions.PipenvOptionsError(
+            "--dev-only",
+            "--dev-only is only permitted in combination with --requirements. "
+            "Aborting."
+        )
     do_lock(
         ctx=ctx,
         clear=state.clear,
         pre=state.installstate.pre,
         keep_outdated=state.installstate.keep_outdated,
         pypi_mirror=state.pypi_mirror,
+        write=not state.quiet,
     )
 
 
@@ -341,13 +379,14 @@ def lock(
     "--fancy",
     is_flag=True,
     default=False,
-    help="Run in shell in fancy mode (for elegantly configured shells).",
+    help="Run in shell in fancy mode. Make sure the shell have no path manipulating"
+         " scripts. Run $pipenv shell for issues with compatibility mode.",
 )
 @option(
     "--anyway",
     is_flag=True,
     default=False,
-    help="Always spawn a subshell, even if one is already spawned.",
+    help="Always spawn a sub-shell, even if one is already spawned.",
 )
 @argument("shell_args", nargs=-1)
 @pypi_mirror_option
@@ -392,7 +431,6 @@ def shell(
 
 
 @cli.command(
-    add_help_option=False,
     short_help="Spawns a command installed into the virtualenv.",
     context_settings=subcommand_context_no_interspersion,
 )
@@ -403,14 +441,14 @@ def shell(
 def run(state, command, args):
     """Spawns a command installed into the virtualenv."""
     from ..core import do_run
-
     do_run(
         command=command, args=args, three=state.three, python=state.python, pypi_mirror=state.pypi_mirror
     )
 
 
 @cli.command(
-    short_help="Checks for security vulnerabilities and against PEP 508 markers provided in Pipfile.",
+    short_help="Checks for PyUp Safety security vulnerabilities and against"
+               " PEP 508 markers provided in Pipfile.",
     context_settings=subcommand_context
 )
 @option(
@@ -419,11 +457,35 @@ def run(state, command, args):
     default=False,
     help="Given a code path, show potentially unused dependencies.",
 )
+@option(
+    "--db",
+    nargs=1,
+    default=lambda: os.environ.get('PIPENV_SAFETY_DB', False),
+    help="Path to a local PyUp Safety vulnerabilities database."
+         " Default: ENV PIPENV_SAFETY_DB or None.",
+)
 @option(
     "--ignore",
     "-i",
     multiple=True,
-    help="Ignore specified vulnerability during safety checks.",
+    help="Ignore specified vulnerability during PyUp Safety checks.",
+)
+@option(
+    "--output",
+    type=Choice(["default", "json", "full-report", "bare"]),
+    default="default",
+    help="Translates to --json, --full-report or --bare from PyUp Safety check",
+)
+@option(
+    "--key",
+    help="Safety API key from PyUp.io for scanning dependencies against a live"
+         " vulnerabilities database. Leave blank for scanning against a"
+         " database that only updates once a month.",
+)
+@option(
+    "--quiet",
+    is_flag=True,
+    help="Quiet standard output, except vulnerability report."
 )
 @common_options
 @system_option
@@ -432,12 +494,16 @@ def run(state, command, args):
 def check(
     state,
     unused=False,
+    db=False,
     style=False,
     ignore=None,
+    output="default",
+    key=None,
+    quiet=False,
     args=None,
     **kwargs
 ):
-    """Checks for security vulnerabilities and against PEP 508 markers provided in Pipfile."""
+    """Checks for PyUp Safety security vulnerabilities and against PEP 508 markers provided in Pipfile."""
     from ..core import do_check
 
     do_check(
@@ -445,7 +511,11 @@ def check(
         python=state.python,
         system=state.system,
         unused=unused,
+        db=db,
         ignore=ignore,
+        output=output,
+        key=key,
+        quiet=quiet,
         args=args,
         pypi_mirror=state.pypi_mirror,
     )
@@ -476,12 +546,14 @@ def update(
         do_sync,
         project,
     )
-
-    ensure_project(three=state.three, python=state.python, warn=True, pypi_mirror=state.pypi_mirror)
+    ensure_project(
+        three=state.three, python=state.python, pypi_mirror=state.pypi_mirror,
+        warn=(not state.quiet), site_packages=state.site_packages, clear=state.clear
+    )
     if not outdated:
         outdated = bool(dry_run)
     if outdated:
-        do_outdated(pypi_mirror=state.pypi_mirror)
+        do_outdated(clear=state.clear, pre=state.installstate.pre, pypi_mirror=state.pypi_mirror)
     packages = [p for p in state.installstate.packages if p]
     editable = [p for p in state.installstate.editables if p]
     if not packages:
@@ -506,12 +578,13 @@ def update(
                     err=True,
                 )
                 ctx.abort()
-
     do_lock(
+        ctx=ctx,
         clear=state.clear,
         pre=state.installstate.pre,
         keep_outdated=state.installstate.keep_outdated,
         pypi_mirror=state.pypi_mirror,
+        write=not state.quiet,
     )
     do_sync(
         ctx=ctx,
@@ -558,7 +631,7 @@ def run_open(state, module, *args, **kwargs):
 
         EDITOR=atom pipenv open requests
     """
-    from ..core import which, ensure_project
+    from ..core import which, ensure_project, inline_activate_virtual_environment
 
     # Ensure that virtualenv is available.
     ensure_project(
@@ -578,6 +651,7 @@ def run_open(state, module, *args, **kwargs):
     else:
         p = c.out.strip().rstrip("cdo")
     echo(crayons.normal("Opening {0!r} in your EDITOR.".format(p), bold=True))
+    inline_activate_virtual_environment()
     edit(filename=p)
     return 0
 
@@ -632,11 +706,9 @@ def sync(
 def clean(ctx, state, dry_run=False, bare=False, user=False):
     """Uninstalls all packages not specified in Pipfile.lock."""
     from ..core import do_clean
-    do_clean(ctx=ctx, three=state.three, python=state.python, dry_run=dry_run)
+    do_clean(ctx=ctx, three=state.three, python=state.python, dry_run=dry_run,
+             system=state.system)
 
 
-# Only invoke the "did you mean" when an argument wasn't passed (it breaks those).
-if "-" not in "".join(sys.argv) and len(sys.argv) > 1:
-    cli = DYMCommandCollection(sources=[cli])
 if __name__ == "__main__":
     cli()
