@@ -4,9 +4,11 @@ from __future__ import absolute_import
 import os
 
 import click.types
+
 from click import (
-    BadParameter, Group, Option, argument, echo, make_pass_decorator, option
+    BadParameter, BadArgumentUsage, Group, Option, argument, echo, make_pass_decorator, option
 )
+from click_didyoumean import DYMMixin
 
 from .. import environments
 from ..utils import is_valid_url
@@ -18,7 +20,7 @@ CONTEXT_SETTINGS = {
 }
 
 
-class PipenvGroup(Group):
+class PipenvGroup(DYMMixin, Group):
     """Custom Group class provides formatted main help"""
 
     def get_help_option(self, ctx):
@@ -54,14 +56,16 @@ class State(object):
         self.index = None
         self.extra_index_urls = []
         self.verbose = False
+        self.quiet = False
         self.pypi_mirror = None
         self.python = None
         self.two = None
         self.three = None
-        self.site_packages = False
+        self.site_packages = None
         self.clear = False
         self.system = False
         self.installstate = InstallState()
+        self.lockoptions = LockOptions()
 
 
 class InstallState(object):
@@ -79,6 +83,11 @@ class InstallState(object):
         self.packages = []
         self.editables = []
 
+class LockOptions(object):
+    def __init__(self):
+        self.dev_only = False
+        self.emit_requirements = False
+        self.emit_requirements_header = False
 
 pass_state = make_pass_decorator(State, ensure=True)
 
@@ -99,7 +108,7 @@ def extra_index_option(f):
         state.extra_index_urls.extend(list(value))
         return value
     return option("--extra-index-url", multiple=True, expose_value=False,
-                  help=u"URLs to the extra PyPI compatible indexes to query for package lookups.",
+                  help=u"URLs to the extra PyPI compatible indexes to query for package look-ups.",
                   callback=callback, envvar="PIP_EXTRA_INDEX_URL")(f)
 
 
@@ -109,8 +118,10 @@ def editable_option(f):
         state.installstate.editables.extend(value)
         return value
     return option('-e', '--editable', expose_value=False, multiple=True,
-                  help='An editable python package URL or path, often to a VCS repo.',
-                  callback=callback, type=click.types.STRING)(f)
+                  callback=callback, type=click.types.STRING, help=(
+                      "An editable Python package URL or path, often to a VCS "
+                      "repository."
+                  ))(f)
 
 
 def sequential_option(f):
@@ -164,16 +175,28 @@ def ignore_pipfile_option(f):
                   callback=callback, type=click.types.BOOL, show_envvar=True)(f)
 
 
-def dev_option(f):
+def _dev_option(f, help_text):
     def callback(ctx, param, value):
         state = ctx.ensure_object(State)
         state.installstate.dev = value
         return value
     return option("--dev", "-d", is_flag=True, default=False, type=click.types.BOOL,
-                  help="Install both develop and default packages.", callback=callback,
+                  help=help_text, callback=callback,
                   expose_value=False, show_envvar=True)(f)
 
 
+def install_dev_option(f):
+    return _dev_option(f, "Install both develop and default packages")
+
+
+def lock_dev_option(f):
+    return _dev_option(f, "Generate both develop and default requirements")
+
+
+def uninstall_dev_option(f):
+    return _dev_option(f, "Deprecated (as it has no effect). May be removed in a future release.")
+
+
 def pre_option(f):
     def callback(ctx, param, value):
         state = ctx.ensure_object(State)
@@ -229,18 +252,39 @@ def verbose_option(f):
     def callback(ctx, param, value):
         state = ctx.ensure_object(State)
         if value:
+            if state.quiet:
+                raise BadArgumentUsage(
+                    "--verbose and --quiet are mutually exclusive! Please choose one!",
+                    ctx=ctx
+                )
             state.verbose = True
-        setup_verbosity(ctx, param, value)
+            setup_verbosity(ctx, param, 1)
     return option("--verbose", "-v", is_flag=True, expose_value=False,
                   callback=callback, help="Verbose mode.", type=click.types.BOOL)(f)
 
 
+def quiet_option(f):
+    def callback(ctx, param, value):
+        state = ctx.ensure_object(State)
+        if value:
+            if state.verbose:
+                raise BadArgumentUsage(
+                    "--verbose and --quiet are mutually exclusive! Please choose one!",
+                    ctx=ctx
+                )
+            state.quiet = True
+            setup_verbosity(ctx, param, -1)
+    return option("--quiet", "-q", is_flag=True, expose_value=False,
+                  callback=callback, help="Quiet mode.", type=click.types.BOOL)(f)
+
+
 def site_packages_option(f):
     def callback(ctx, param, value):
         state = ctx.ensure_object(State)
+        validate_bool_or_none(ctx, param, value)
         state.site_packages = value
         return value
-    return option("--site-packages", is_flag=True, default=False, type=click.types.BOOL,
+    return option("--site-packages/--no-site-packages", is_flag=True, default=None,
                   help="Enable site-packages for the virtualenv.", callback=callback,
                   expose_value=False, show_envvar=True)(f)
 
@@ -276,15 +320,32 @@ def requirementstxt_option(f):
                   help="Import a requirements.txt file.", callback=callback)(f)
 
 
-def requirements_flag(f):
+def emit_requirements_flag(f):
     def callback(ctx, param, value):
         state = ctx.ensure_object(State)
         if value:
-            state.installstate.requirementstxt = value
+            state.lockoptions.emit_requirements = value
         return value
     return option("--requirements", "-r", default=False, is_flag=True, expose_value=False,
                   help="Generate output in requirements.txt format.", callback=callback)(f)
 
+def emit_requirements_header_flag(f):
+    def callback(ctx, param, value):
+        state = ctx.ensure_object(State)
+        if value:
+            state.lockoptions.emit_requirements_header = value
+        return value
+    return option("--header/--no-header", default=True, is_flag=True, expose_value=False,
+                  help="Add header to generated requirements", callback=callback)(f)
+
+def dev_only_flag(f):
+    def callback(ctx, param, value):
+        state = ctx.ensure_object(State)
+        if value:
+            state.lockoptions.dev_only = value
+        return value
+    return option("--dev-only", default=False, is_flag=True, expose_value=False,
+                  help="Emit development dependencies *only* (overrides --dev)", callback=callback)(f)
 
 def code_option(f):
     def callback(ctx, param, value):
@@ -292,8 +353,9 @@ def code_option(f):
         if value:
             state.installstate.code = value
         return value
-    return option("--code", "-c", nargs=1, default=False, help="Import from codebase.",
-                  callback=callback, expose_value=False)(f)
+    return option("--code", "-c", nargs=1, default=False, help="Install packages "
+                  "automatically discovered from import statements.", callback=callback,
+                  expose_value=False)(f)
 
 
 def deploy_option(f):
@@ -310,8 +372,14 @@ def setup_verbosity(ctx, param, value):
     if not value:
         return
     import logging
-    logging.getLogger("pip").setLevel(logging.INFO)
-    environments.PIPENV_VERBOSITY = 1
+    loggers = ("pip", "piptools")
+    if value == 1:
+        for logger in loggers:
+            logging.getLogger(logger).setLevel(logging.INFO)
+    elif value == -1:
+        for logger in loggers:
+            logging.getLogger(logger).setLevel(logging.CRITICAL)
+    environments.PIPENV_VERBOSITY = value
 
 
 def validate_python_path(ctx, param, value):
@@ -326,6 +394,12 @@ def validate_python_path(ctx, param, value):
     return value
 
 
+def validate_bool_or_none(ctx, param, value):
+    if value is not None:
+        return click.types.BOOL(value)
+    return False
+
+
 def validate_pypi_mirror(ctx, param, value):
     if value and not is_valid_url(value):
         raise BadParameter("Invalid PyPI mirror URL: %s" % value)
@@ -343,7 +417,6 @@ def common_options(f):
 
 def install_base_options(f):
     f = common_options(f)
-    f = dev_option(f)
     f = pre_option(f)
     f = keep_outdated_option(f)
     return f
@@ -351,6 +424,7 @@ def install_base_options(f):
 
 def uninstall_options(f):
     f = install_base_options(f)
+    f = uninstall_dev_option(f)
     f = skip_lock_option(f)
     f = editable_option(f)
     f = package_arg(f)
@@ -359,12 +433,15 @@ def uninstall_options(f):
 
 def lock_options(f):
     f = install_base_options(f)
-    f = requirements_flag(f)
+    f = lock_dev_option(f)
+    f = emit_requirements_flag(f)
+    f = dev_only_flag(f)
     return f
 
 
 def sync_options(f):
     f = install_base_options(f)
+    f = install_dev_option(f)
     f = sequential_option(f)
     return f
 
@@ -374,7 +451,6 @@ def install_options(f):
     f = index_option(f)
     f = extra_index_option(f)
     f = requirementstxt_option(f)
-    f = pre_option(f)
     f = selective_upgrade_option(f)
     f = ignore_pipfile_option(f)
     f = editable_option(f)
